#include <iostream>
#include <fstream>
#include <cstdio>
#include <unistd.h>
#include <fcntl.h>
#include <ctime>

#include "fun.h"
#include "isdn.hh"
#include "config.h"

using namespace std;

void isdn::rejectcall()
{
    sendah("ATH0\n");
}

void isdn::acceptcall()
{
    sendah("ATA\n");
}

void isdn::beep(unsigned len, float vol)
{
	int val1 = (int)(-3000*vol);
	int val2 = (int)( 3000*vol);
    string s;
    while(len--)s += lin2mu(((len/4) % 4) ? val1 : val2);
    sendah(s);
}

void isdn::pipeout(const string &cmd)
{
    FILE *fp = popen(cmd.c_str(), "r");
    if(!fp)
    {
        cerr << "isdn: popen failed!\n";
        return;
    }
    string s;
    for(;;)
    {
        int c = fgetc(fp);
        if(c < 0)break;
        
        if(c==DLE)s += char(c);
        s += char(c);
    }
    sendah(s);
    pclose(fp);
}

void isdn::readandsend(class db &msgdb, cidtype &caller)
{
    sendah("AT+VRX+VTX\n");
    
    beep(1600, 0); // 200 ms of silence
    
    switch(caller.belltype)
    {
        case 0: break;
        case 1:
            beep(6000);
            break;
        case 2:
            // FIXME. This is just a default value now...
            caller.bell = "jt viesti, saat sen shkpostilla. mys nppily sallittu";
            pipeout("/usr/src/rsynth-2.0/say -a -x 1400 -c 3 -S 0.65 -o - '" + caller.bell + "'");
            break;
        case 3:
        	pipeout("cat " VOICEBOXDIR "/b5.au");
        	break;
    }

    message msg(msgdb);
    
    msg.recordtime = time(0);
    
    char TimeBuf[64];
    sprintf(TimeBuf, "%lu", msg.recordtime);
    
    string filename = SPOOLDIR "/";
    filename += TimeBuf;
    filename += '-';
    filename += caller.email;
    
    ofstream fp(filename.c_str());
    
    fp << ".snd";
    msg.voicedata=".snd";
    fp.put(char(0));   // header size
    fp.put(char(0));
    fp.put(char(0));
    fp.put(char(24));
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)24;
    fp.put(char(255)); // size - unspecified
    fp.put(char(255));
    fp.put(char(255));
    fp.put(char(255));
    msg.voicedata+=(char)255;
    msg.voicedata+=(char)255;
    msg.voicedata+=(char)255;
    msg.voicedata+=(char)255;
    fp.put(char(0));   // encoding
    fp.put(char(0));
    fp.put(char(0));
    fp.put(char(1));
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)1;
    fp.put(char(0));   // rate
    fp.put(char(0));
    fp.put(char(8000/256));
    fp.put(char(8000%256));
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)(8000/256);
    msg.voicedata+=(char)(8000%256);
    fp.put(char(0));   // channels
    fp.put(char(0));
    fp.put(char(0));
    fp.put(char(1));    
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)0;
    msg.voicedata+=(char)1;
    
    msg.pathname   = filename;
    msg.email      = caller.email;
    msg.cid        = caller.cid;
    
    for(;;)
    {
        char c = getc();
        if(c == DLE)
        {
            c = getc();
            if(c == ERX || c == ETX)
            {
                break;
            }
            if(c != DLE)
            {
                cerr << "isdn: Got DTMF code: " << c << endl;
                msg.numberdata += c;
                continue;
            }
        }
        fp.put(c);
        msg.voicedata += c;
    }
    
    msg.bytelength      = msg.voicedata.size();
    msg.timelength      = time(0) - msg.recordtime;
    msg.compressiontype = "";
    msg.mimetype        = "audio/basic";
    
    cerr << "isdn: Ending recording.\n";
    
    if(msg.insert() < 0)
        cerr << "isdn: Failed inserting to database!\n";
    else
        cerr << "isdn: Inserted to database.\n";
}

void isdn::open(const string &devicefile)
{
    fd = std::open(devicefile.c_str(), O_RDWR);
}    

void isdn::init()
{
    sendah(
        "ATZ\n"         // reset
        "ATI1"          // info :)
         " E1"          // ok mimic me if you want.
         " +FCLASS=8"   // enter phone(voice) mode
         " +VIP"        // reset voice params (must be done after +flass=8)
         " S0=0"        // no automatic answers
         " +VSM=6"      // 8-bit uLaw-mode 8kHz
        "\n");
}

void isdn::setnumber(const string &num)
{
    sendah("AT&E" + num +
            " &L" + num + "\n");
}

void isdn::loop(class db &userdb, class db &msgdb)
{
    if(fd < 0)return;
    
    string callernum;
    cidtype tmp(userdb);
    

    cerr << "isdn: Listening.\n";
    
    for(;;)
    {
        string s;
        getline(s);
        if(s.size())
	        cerr << "isdn: Got: " << s << endl;
        
        if(s.substr(0, 13) == "CALLER NUMBER")
        {
            unsigned a, b=s.size();
            for(a=0; a<b; ++a)if(s[a] == ':')break;
            if(++a < b)while(s[a] == ' ')++a;
            callernum = s.substr(a);
            cerr << "isdn: Caller: " << callernum << endl;
        }
        if(s == "RING")
        {
            cerr << "isdn: RING from " << callernum << " - " << std::flush;
            if(tmp.find(callernum) < 0)
            {
                rejectcall();
                cerr << "rejected!\n";
            }
            else
            {
                acceptcall();
                cerr << "accepted.\n";
            }
        }
        if(s == "VCON")
        {
            cerr << "isdn: Connection established, recording for " << tmp.email << ".\n";
            readandsend(msgdb, tmp);
        }
        if(s == "NO CARRIER")
        {
            cerr << "isdn: Connection lost. Good.\n";
        }
    }
}
